-- version: 0.3.3

---0.1.0
-- buttons
-- menu, which is struct of buttons (and other components)

---0.2.0
-- checkbox

---0.3.0
-- textbox (with calling filling subroutine with keyboards)

---0.4.0
-- TODO: value slider (sliding between min and max value)

-- TODO: dropdown menu (<select>)
-- TODO: select menu (listbox of buttons with scrollbar)
-- TODO: textarea (with scrollbar)

module(...,package.seeall);

--- includes
--package.loaded.Keyboard = nil;
--require "Keyboard";

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
-------------------------------[gui macros]-------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
BUTTONCOLOR1 = color.new(150,150,150)    -- middle of button
BUTTONCOLOR2 = color.new(200,200,200)    -- upper and left edge color
BUTTONCOLOR3 = color.new(100,100,100)    -- lower and right edge color
BUTTONFONTSIZE = 20                      -- size of font
HINTFONTSIZE = 15                        -- size of font for helping lines
BUTTONFONTCOLOR = color.new(0,0,0)	  -- color of button text
BUTTONFONTCOLOR2 = color.new(30,30,30)   -- color of button text when pressed
BUTTONLINEWIDTH = 1                      -- width of line edge of buttons
BUTTONDISCYCLES = 0					        -- number of render calls before button is again unpressed

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
----------------------[helping render function functions]-----------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

function clear(color1)
-- clearing content to black
	screen.fillrect(0,0,screen.width(),screen.height(),color1)
end

function renderBrick(x,y,w,h)
   -- button isnt pressed
	screen.fillrect(x,y,w-1,h-1,BUTTONCOLOR1)
	screen.drawline(x,y,x+w-1,y,BUTTONCOLOR2,BUTTONLINEWIDTH)
	screen.drawline(x,y,x,y+h-1,BUTTONCOLOR2,BUTTONLINEWIDTH)
	screen.drawline(x+w-1,y,x+w-1,y+h-1,BUTTONCOLOR3,BUTTONLINEWIDTH)
	screen.drawline(x,y+h-1,x+w-1,y+h-1,BUTTONCOLOR3,BUTTONLINEWIDTH)
end

function renderReversedBrick(x,y,w,h)
   -- pressed button
   screen.fillrect(x,y,w-1,h-1,BUTTONFONTCOLOR)
   screen.drawline(x,y,x+w-1,y,BUTTONCOLOR3,BUTTONLINEWIDTH)
   screen.drawline(x,y,x,y+h-1,BUTTONCOLOR3,BUTTONLINEWIDTH)
   screen.drawline(x+w-1,y,x+w-1,y+h-1,BUTTONCOLOR2,BUTTONLINEWIDTH)
   screen.drawline(x,y+h-1,x+w-1,y+h-1,BUTTONCOLOR2,BUTTONLINEWIDTH)
end

function renderBorder(x,y,w,h)
   -- outer edge
	screen.drawline(x,y,x+w-1,y,BUTTONCOLOR2,BUTTONLINEWIDTH)
   screen.drawline(x,y,x,y+h-1,BUTTONCOLOR2,BUTTONLINEWIDTH)
   screen.drawline(x+w-1,y,x+w-1,y+h-1,BUTTONCOLOR3,BUTTONLINEWIDTH)
   screen.drawline(x,y+h-1,x+w-1,y+h-1,BUTTONCOLOR3,BUTTONLINEWIDTH)
   -- between edges
   screen.drawrect(x+1,y+1,x+w-2,y+h-2,BUTTONCOLOR1)
   screen.drawrect(x+2,y+2,x+w-3,y+h-3,BUTTONCOLOR1)
	-- inner edge
   screen.drawline(x+3,y+3,x+w-4,y+3,BUTTONCOLOR3,BUTTONLINEWIDTH)
   screen.drawline(x+3,y+3,x+3,y+h-4,BUTTONCOLOR3,BUTTONLINEWIDTH)
   screen.drawline(x+w-4,y+3,x+w-4,y+h-4,BUTTONCOLOR2,BUTTONLINEWIDTH)
   screen.drawline(x+3,y+h-4,x+w-4,y+h-4,BUTTONCOLOR2,BUTTONLINEWIDTH)
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
------------------------------[button functions]--------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

--{id of button, shown name, x of left upper edge}
--style = 0 ... button without image, just with text
--style = 1 ... button with only image
--style = 2 ... button with image and text

function changeButton(btn)
   -- disabling
	btn.used = BUTTONDISCYCLES

   -- calling button function (with button as parameter)
	local meOut = nil;
	if(btn.func ~= nil) then
      meOut = btn.func(btn);
   end;

   -- return back stuff from function in button
   return(meOut)
end

function buttonCheck(x,y,button,xShift,yShift)
	-- checking if button is pressed (disabled for a while)
	--if(button.used > 0) then
	--	return(0)
	--end
	
	-- checking if button is on this x,y
	if( x> xShift+button.x
	and x< xShift+button.x+button.width-1
	and y> yShift+button.y
	and y< yShift+button.y+button.height-1) then
		return(1)
	end
	return(0)
end

function buttonRender(btn,xShift,yShift)
   -- shifted position
   local x = xShift+btn.x
   local y = yShift+btn.y

   -- font size and color
   local osize = text.size(BUTTONFONTSIZE)
	text.color(BUTTONFONTCOLOR)

   -- different styles of rendering
	if(btn.style==1 and btn.pix ~= nil) then
		-- style 1 ... only picture
		if(btn.used > 0) then
         btn.pix:draw(x,y,btn.width-1,btn.height-1)
      else
         if(btn.pix2 ~= nil) then
            btn.pix2:draw(x,y,btn.width-1,btn.height-1);
         else
            btn.pix:draw(x,y,btn.width-1,btn.height-1);
         end;
      end;
	elseif(btn.style==2 and btn.pix ~= nil) then
		-- style 2 ... picture and text
      if(btn.used > 0) then
         if(btn.pix2 ~= nil) then
            btn.pix2:draw(x,y,btn.width-1,btn.height-1);
         else
            btn.pix:draw(x,y,btn.width-1,btn.height-1);
         end;
         text.color(BUTTONFONTCOLOR2)
      else
         btn.pix:draw(x,y,btn.width-1,btn.height-1)
         text.color(BUTTONFONTCOLOR)
      end;
      -- writing caption (text from btn.name)
      text.draw(x,y+(btn.height-BUTTONFONTSIZE)/2,btn.name, "center", x+btn.width)
	else
		-- style 0 ... win graphics and text only
		if(btn.used > 0) then
			-- pressed buton
			renderReversedBrick(x,y,btn.width,btn.height);
         text.color(BUTTONCOLOR1)
		else
         -- button isnt pressed
         renderBrick(x,y,btn.width,btn.height);
		   text.color(BUTTONFONTCOLOR)
      end
      -- writing caption (text from btn.name)
      text.draw(x,y+(btn.height-BUTTONFONTSIZE)/2,btn.name, "center", x+btn.width)
   end
	
	-- decreasing dissable countdown
   btn.used = btn.used - 1

	-- set back font size
	text.size(osize)
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
------------------------------[checkbox functions]------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function getCheck(btn)
   if(btn.used == 1) then
      return(true)
   else
      return(false)
   end
end

function changeCheckbox(btn)
   -- state in field "func"
   state = btn.func

   -- change state
   if(btn.used > 0) then
      btn.used = 0
      btn.func = false
   else
      btn.used = 1
      btn.func = true
   end

   -- return back 0
   return(0)
end

function checkboxRender(btn,xShift,yShift)
   -- shifted position
   local x = xShift+btn.x
   local y = yShift+btn.y

   -- font size and color
   local osize = text.size(BUTTONFONTSIZE)
	text.color(BUTTONFONTCOLOR)

	-- style 0 ... win graphics and text only
	
	-- border
	renderBorder(x,y,btn.height,btn.height)
      
	if(btn.used > 0) then
      -- checkbox is checked (insert small rectangle)
      renderBrick(x+10,y+10,btn.height-20,btn.height-20)
   end

   -- writing caption (text from btn.name)
   text.color(BUTTONCOLOR1)
   text.draw(x+btn.height+5,y+(btn.height-BUTTONFONTSIZE)/2,btn.name, "left",x+btn.width-5)

	-- set back font size
	text.size(osize)
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
------------------------------[textbox functions]-------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function getText(btn)
   if(type(btn.func)=="string") then
      return(btn.func)
   else
      return("")
   end
end

function renderTextboxScreenInput(btn)
   -- clearing
   clear(black)

   -- font size and color
   local osize = text.size(HINTFONTSIZE)
	text.color(BUTTONCOLOR1)

	-- help caption
	text.draw(0,1,"stuff","center",400)

	-- frame for text
	xi = 5
	yi = BUTTONFONTSIZE
	wighti = 400-2*xi
	heighti = BUTTONFONTSIZE*2
	renderBorder(xi,yi,wighti,heighti)

   -- writing text inside box
   text.size(BUTTONFONTSIZE)
   text.color(BUTTONCOLOR1)
   text.draw(xi+5,yi+(heighti-BUTTONFONTSIZE)/2,btn.func, "left")

   -- hint
   local osize = text.size(HINTFONTSIZE)
	text.draw(0,heighti+HINTFONTSIZE+5,"home button: click to change keyboard, hold to end","center",400)

   -- set back font size
	text.size(osize)
end

function changeTextbox(btn)
   -- init of string
   if(type(btn.func)~="string") then
      btn.func = ""
   end

   -- string for new text
   local butText = btn.fnc
   local charb

   -- keyboards switching
   local drawFunc = Keyboard.charboardRender
   local clickFunc = Keyboard.charboardFunction
   local keyboardIndex = 0 -- charboard

   -- its own render and button cycle
   while true do
      if(control.read()==1) then
         -- touch screen event
		   if(control.isTouch()==1) then
    			if(touch.click()==1) then
    				x,y = touch.pos()
      			-- checking for keyboard buttons
      			charb = clickFunc(x,y)
               if(charb == Keyboard.BUTTON_BACKSPACE) then
      				-- BACKSPACE
      				btn.func = string.sub(btn.func,1,string.len(btn.func)-1)
               elseif(charb == nil
               or charb == Keyboard.BUTTON_RIGHT
               or charb == Keyboard.BUTTON_LEFT
               or charb == Keyboard.BUTTON_UP
               or charb == Keyboard.BUTTON_DOWN) then
                  -- do nothing now
               elseif(charb == Keyboard.BUTTON_ENTER) then
                  -- enter, go back out
                  break
      			else
      				-- char input (append it at the end)
                  btn.func = btn.func..charb
      			end
    			end
    		-- player buttom has been pressed
    		elseif control.isButton()==1 then
    		    -- player home buttom event
		       -- and home button has been clicked
			    if(button.home() == 1) then
                 if(button.click()==1) then
                    -- change keyboard
                    if(keyboardIndex == 0) then
                        keyboardIndex = 1
                        drawFunc = Keyboard.capsboardRender
                        clickFunc = Keyboard.capsboardFunction
                     elseif(keyboardIndex == 1) then
                        drawFunc = Keyboard.numboardRender
                        clickFunc = Keyboard.numboardFunction
                        keyboardIndex = 2
                     elseif(keyboardIndex == 2) then
                        drawFunc = Keyboard.specboardRender
                        clickFunc = Keyboard.specboardFunction
                        keyboardIndex = 3
                     else
                        drawFunc = Keyboard.charboardRender
                        clickFunc = Keyboard.charboardFunction
                        keyboardIndex = 0
                     end
                  elseif(button.hold()==1) then
                   break
				     end
			    end
		    end
      else
         -- render interface
         -- upper part
         renderTextboxScreenInput(btn)

         -- keyboard
         drawFunc()

         -- return button (end edit)
         --buttonRender(retbtn,0,0)

         -- updating
         screen.update()
         os.wait(10)
      end
   end

   return(0)
end

function textboxRender(btn,xShift,yShift)
   -- shifted position
   local x = xShift+btn.x
   local y = yShift+btn.y

   -- font size and color
   local osize = text.size(BUTTONFONTSIZE)
	text.color(BUTTONFONTCOLOR)

	-- style 0 ... win graphics and text only
	renderBorder(x,y,btn.width,btn.height)

   -- writing caption (text from btn.name)
   text.color(BUTTONCOLOR1)
   text.draw(x+5,y+(btn.height-BUTTONFONTSIZE)/2,btn.func, "left")

	-- set back font size
	text.size(osize)
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------[menu functions]--------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function menuCheck(x,y, menu)
   -- shifted beginning of menu, if set
   local xShift0 = 0
   local yShift0 = 0
   
   -- shifted x of beginning
   if(menu.xShift ~= nil) then
      xShift0 = menu.xShift
   end
   
   -- shifted y of beginning
   if(menu.yShift ~= nil) then
      yShift0 = menu.yShift
   end
   
   -- Shift optimalization cant be done here,
   -- because menu and button position can be shifted
   -- out of bounds of screen.
   -- Shift optimization mut be called before calling
   -- menuCheck function.
   
   -- check for buttonds in that area
   for i=1,#menu.buttons do
      if(buttonCheck(x,y,menu.buttons[i],xShift0,yShift0)==1) then
         return(i)
      end
   end
   
   -- return zero index (not pressed any button)
   return(0)
end

function menuRender(menu)
   -- shifted beginning of menu, if set
   local xShift0 = 0
   local yShift0 = 0

   -- shifted x of beginning
   if(menu.xShift ~= nil) then
      xShift0 = menu.xShift
   end

   -- shifted y of beginning
   if(menu.yShift ~= nil) then
      yShift0 = menu.yShift
   end

   -- rendering buttons
   for i=1,#menu.buttons do
      if(menu.buttons[i].style < 10) then
         buttonRender(menu.buttons[i],xShift0,yShift0)
         
      elseif(menu.buttons[i].style < 20) then
         checkboxRender(menu.buttons[i],xShift0,yShift0)

      elseif(menu.buttons[i].style < 30) then
         textboxRender(menu.buttons[i],xShift0,yShift0)

      end
   end
end

function menuFunction(x,y,menu)
	--if(touch.event() == 1) then -- probably clears buffer for imput like rest of touch.stuff
		--x,y = touch.pos()
	   -- click
		if(touch.click()== 1) then
			print("click")
			local index = menuCheck(x,y,menu)
			print("menu button number: "..tostring(index))
			if(index > 0) then
            -- button
            if(menu.buttons[index].style < 10) then
               return(changeButton(menu.buttons[index]))
            -- checkbox
            elseif(menu.buttons[index].style < 20) then
               return(changeCheckbox(menu.buttons[index]))
            -- textbox
            else
               return(changeTextbox(menu.buttons[index]))
            end
			end
			
		-- touch up
		elseif(touch.up()==1) then
			print("touch up")
		--	return(0)
		
		-- touch down
		elseif(touch.down()==1) then
			print("touch down")
		--	return(0)
		
		-- touch hold
		elseif(touch.hold()==1) then
		-- check only last used key if held (last touchdowned)
			print("touch hold")
		--	return(0)
		
		-- touch move
		elseif(touch.move()==1) then
			print("touch move")
		--	return(0)
		end
	--end

	-- return no action
	return(nil)
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
------------------------------[dilogbox functions]------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
